/*
 * Copyright 2013 Canonical Ltd.
 *
 * This file is part of ubuntu-calculator-app.
 *
 * ubuntu-calculator-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * ubuntu-calculator-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import Ubuntu.Components 0.1

Item {
    width: parent.width
    height: grid.height+units.gu(4)

    property int calcGridUnit: width / 50


    Text {
        visible: formulaView.__wasAtYBegining && formulaView.__displaceDist > units.gu(2)
        font.pixelSize: units.gu(2)
        text: formulaView.__toBeRefresh ? i18n.tr("Release to start new calculation") : i18n.tr("Pull to start new calculation")
        font.family: "Ubuntu"
        anchors.horizontalCenter: parent.horizontalCenter
        color: '#757373'
    }

    Rectangle {
        width: parent.width
        height: grid.height+units.gu(4)
        color: "#ECECEC"

        anchors{
            top: parent.top
            topMargin: formulaView.__wasAtYBegining & formulaView.__displaceDist > 0 ? formulaView.__displaceDist : 0
        }

        Item {
            id: grid
            width: (calcGridUnit*9)*4 + (calcGridUnit*2)*3
            height: (calcGridUnit*6)*5 + (calcGridUnit*2)*4

            anchors{
                horizontalCenter: parent.horizontalCenter
                top: parent.top
                topMargin: units.gu(2)
            }

            KeyboardButton {
                objectName: "clearButton"
                id: clearButton
                x: 0
                y: 0
                // TRANSLATORS: Refers to Clear, keep the translation to 2 characters
                text: i18n.tr("C")
                textColor: "#464D6F"
                buttonColor: "#E4E4E7"
                pressedColor: "#D4D4D7"
                onClicked: {
                    formulaPop();
                }
            }

            KeyboardButton {
                objectName: "signButton"
                x: (calcGridUnit*11)
                y: 0
                text: "+/-"
                onClicked: {
                    changeSign();
                }
            }

            KeyboardButton {
                objectName: "divideButton"
                x: (calcGridUnit*11)*2
                y: 0
                text: "÷"
                onClicked: formulaPush("/")
            }

            KeyboardButton {
                objectName: "multiplyButton"
                x: (calcGridUnit*11)*3
                y: 0
                text: "×"
                onClicked: formulaPush("*")
            }

            KeyboardButton {
                objectName: "sevenButton"
                x: 0
                y: (calcGridUnit*8)
                text: Number(7).toLocaleString(Qt.locale(), "f", 0)
                onClicked: formulaPush(text)
            }

            KeyboardButton {
                objectName: "eightButton"
                x: (calcGridUnit*11)
                y: (calcGridUnit*8)
                text: Number(8).toLocaleString(Qt.locale(), "f", 0)
                onClicked: formulaPush(text)
            }

            KeyboardButton {
                objectName: "nineButton"
                x: (calcGridUnit*11)*2
                y: (calcGridUnit*8)
                text: Number(9).toLocaleString(Qt.locale(), "f", 0)
                onClicked: formulaPush(text)
            }

            KeyboardButton {
                objectName: "minusButton"
                x: (calcGridUnit*11)*3
                y: (calcGridUnit*8)
                text: "−"
                onClicked: formulaPush('-')
            }

            KeyboardButton {
                objectName: "fourButton"
                x: 0
                y: (calcGridUnit*8)*2
                text: Number(4).toLocaleString(Qt.locale(), "f", 0)
                onClicked: formulaPush(text)
            }

            KeyboardButton {
                objectName: "fiveButton"
                x: (calcGridUnit*11)
                y: (calcGridUnit*8)*2
                text: Number(5).toLocaleString(Qt.locale(), "f", 0)
                onClicked: formulaPush(text)
            }

            KeyboardButton {
                objectName: "sixButton"
                x: (calcGridUnit*11)*2
                y: (calcGridUnit*8)*2
                text: Number(6).toLocaleString(Qt.locale(), "f", 0)
                onClicked: formulaPush(text)
            }

            KeyboardButton {
                objectName: "plusButton"
                x: (calcGridUnit*11)*3
                y: (calcGridUnit*8)*2
                text: "+"
                onClicked: formulaPush(text)
            }

            KeyboardButton {
                objectName: "oneButton"
                x: 0
                y: (calcGridUnit*8)*3
                text: Number(1).toLocaleString(Qt.locale(), "f", 0)
                onClicked: formulaPush(text)
            }

            KeyboardButton {
                objectName: "twoButton"
                x: (calcGridUnit*11)
                y: (calcGridUnit*8)*3
                text: Number(2).toLocaleString(Qt.locale(), "f", 0)
                onClicked: formulaPush(text)
            }

            KeyboardButton {
                objectName: "threeButton"
                x: (calcGridUnit*11)*2
                y: (calcGridUnit*8)*3
                text: Number(3).toLocaleString(Qt.locale(), "f", 0)
                onClicked: formulaPush(text)
            }

            KeyboardButton {
                objectName: "equalsButton"
                x: (calcGridUnit*11)*3
                y: (calcGridUnit*8)*3
                height: (calcGridUnit*14)
                text: "="
                textColor: "#464D6F"
                buttonColor: "#E4E4E7"
                pressedColor: "#D4D4D7"
                onClicked: {
                    calculate();
                }
            }

            KeyboardButton {
                objectName: "zeroButton"
                x: 0
                y: (calcGridUnit*8)*4
                width: (calcGridUnit*20)
                text: Number(0).toLocaleString(Qt.locale(), "f", 0)
                onClicked: formulaPush(text)
            }

            KeyboardButton {
                objectName: "pointButton"
                x: (calcGridUnit*11)*2
                y: (calcGridUnit*8)*4
                text: "."
                onClicked: formulaPush(text)
            }
        }
    }

    states: [
        State {
            name: "calculationCompleted"
            PropertyChanges {
                target: clearButton
                // TRANSLATORS: Refers to All Clear, keep the translation to 2 characters
                text: i18n.tr("AC")
                onClicked: {
                    clear();
                }
            }
        }
    ]

    onStateChanged: {
        console.log(state);
    }
}
