/*
 * Copyright 2013 Canonical Ltd.
 *
 * This file is part of ubuntu-calculator-app.
 *
 * ubuntu-calculator-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * ubuntu-calculator-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

Qt.include("engine.js");

var context = new Context;

function Formula() {
    var previous = new Token(0, T_NUMBER);
    var formula = '';
    var lookupTbl = {
        '-': '−',
        '/': '÷',
        '*': '×'
    };
    var table = {
        "65" : "+",
        "66" : "-",
        "67" : "*",
        "68" : "/"
    }

    var isLastResult = false;   // This var becames true if the last button pressed by user is equal
    var isChangeSign = false;   // This var becames true if user want to change the sign of a result

    var _calculate = function(func){
        try{
            var result = parse(func, context);
            if (result === Number.POSITIVE_INFINITY)
                result = '∞';
            else if (result === Number.NEGATIVE_INFINITY)
                result = '-∞';
        }catch(exception){
            if(exception instanceof DivisionByZeroError){
                result = "division by zero error";
            } else{
                throw new SyntaxError("malformed calculation");
            }
        }
        return result;
    }

    this.push = function(digit){
        var last = (new Scanner(previous.value.toString()+digit, context)).tokens.pop();

        if (!isNaN(digit) && isLastResult) return;  // After a calc you have to add a sign before add a number

        switch(last.type){
        case T_PLUS:
        case T_MINUS:
        case T_DIV:
        case T_TIMES:
            if (previous.type & T_OPERATOR) return; // Not two operator below
            break;
        case T_UNARY_PLUS:
        case T_UNARY_MINUS:
            if(previous.type === T_UNARY_MINUS || previous.type === T_UNARY_PLUS ) return;
            break;
        }

        formulaView.headerItem.state = "newPush"; // This is to change "AC" to "C" when user continue a calc

        /* After user press equal the operation we have to forget the math order
         * and restart the operation from the result, 
         * but set formula = result is impossible, 
         * because we lost a lot of precious information
         */
        if (isLastResult)
            formula = "(" + formula + ")";

        formula += digit;
        isLastResult = false;
        previous = last;
        var ret = {value: '', type: last.type};
        //formula Shouldn't start with operators.
        var firstChar = formula.substring(0,1);
        if(isNaN(firstChar) && firstChar !== "(" && firstChar !== "-"){ // First letter should be digit or open bracket or minus
            ret.type = T_OPERATOR // Return
            formula = ""; // Make formula var empty
        }
        if(lookupTbl[digit] !== undefined){
            ret.value = lookupTbl[digit];
        } else {
            ret.value = digit;
        }
        return ret;
    };

    this.pop = function(){
        var scannedFormula = new Scanner(formula, context);
        var last = {type:T_NUMBER};
        var removeSize = 0;
        while(scannedFormula.tokens.length > 0 && (last.type === T_NUMBER || last.type === T_UNARY_PLUS || last.type === T_UNARY_MINUS)){
            last = scannedFormula.tokens.pop();
            removeSize += last.value.toString().length
        }
        formula = formula.substring(0, formula.length - removeSize);
        previous = scannedFormula.tokens.pop();
    };

    this.calculate = function(){
        var result;
        var previousSize = previous.value.toString().length;
        if(formula.length <= previousSize) return;
        var scannedFormula = new Scanner(formula, context);

        // Last and secondLast are one a number and the other an operator. Both can be both, it depends on user input
        var last = scannedFormula.tokens.pop();
        var secondLast = scannedFormula.tokens.pop();

        // In this case user press equal twice E.G. 2*2=4=8
        if (isLastResult && !isChangeSign && table[secondLast.type] !== undefined)
            formula = '(' + formula + ')' + table[secondLast.type] + last.value.toString();
        // In this case user press a number, and operator (or only an operator, if it isn't the first calc), and then equal
        else if (last.type & T_OPERATOR)
            formula = formula + secondLast.value.toString();

        result = _calculate(formula);

        // If user want to change sign to the result
        if (isLastResult && isChangeSign) {
            // TO-DO: check if the user has change the sign yet, and remove old *-1
            result = -result;   // Change the sign
            formula = '(' + formula + ')*-1'; // Remember the change of the sign for the future
        }

        isLastResult = true;
        return result;
    }

    this.changeSign = function(){

        // If user want to change the sign of the result
        if (isLastResult) {
            isChangeSign = true;
            screenFormula.pop(); // Delete last result from the screen
            calculate();
            isChangeSign = false;
            return;
        }

        switch(previous.type){
        case T_PLUS:
        case T_MINUS:
        case T_DIV:
        case T_TIMES:
            // After a sign, push minus
            formulaPush("-");
            break;
        default:
            var scannedFormula = new Scanner(formula, context);
            var last = {type: undefined};
            var toAdd = [];
            var addMinus = true;

            // Create a queue with all element to re-add to var formula
            while(scannedFormula.tokens.length > 0 && last.type !== T_PLUS &&
                  last.type !== T_MINUS &&
                  last.type !== T_DIV &&
                  last.type !== T_TIMES ){
                last = scannedFormula.tokens.pop();
                if(last.type !== T_UNARY_MINUS && last.type !== T_UNARY_PLUS){
                    toAdd.push(last);
                } else {
                    if(last.type === T_UNARY_MINUS)
                        addMinus = false;
                }
            }

            formulaPop();

            while(toAdd.length > 0){
                var el = toAdd.pop()
                formulaPush(el.value.toString());
                if((el.type & T_OPERATOR) && addMinus){
                    formulaPush('-');
                }
            }
        }
    }

    this.hasResults = function(){
        if(results.length > 0){
            return true;
        }
        return false;
    }
}
